@@ -36,8 +36,8 @@ LuaConfigItems::LuaConfigItems()
 {
   DNSName root("."); // don't use g_rootdnsname here, it might not exist yet
   for (const auto& dsRecord : rootDSs) {
-    auto ds = std::dynamic_pointer_cast<DSRecordContent>(DSRecordContent::make(dsRecord));
-    dsAnchors[root].insert(*ds);
+    auto dsRecContent = std::dynamic_pointer_cast<DSRecordContent>(DSRecordContent::make(dsRecord));
+    dsAnchors[root].emplace(*dsRecContent);
   }
 }
 
@@ -99,114 +99,115 @@ typename C::value_type::second_type constGet(const C& c, const std::string& name
 
 typedef std::unordered_map<std::string, boost::variant<bool, uint32_t, std::string, std::vector<std::pair<int, std::string>>>> rpzOptions_t;
 
-static void parseRPZParameters(rpzOptions_t& have, std::shared_ptr<DNSFilterEngine::Zone>& zone, std::string& polName, boost::optional<DNSFilterEngine::Policy>& defpol, bool& defpolOverrideLocal, uint32_t& maxTTL)
+static void parseRPZParameters(const rpzOptions_t& have, RPZTrackerParams& params)
 {
   if (have.count("policyName") != 0) {
-    polName = boost::get<std::string>(have["policyName"]);
+    params.polName = boost::get<std::string>(have.at("policyName"));
   }
   if (have.count("defpol") != 0) {
-    defpol = DNSFilterEngine::Policy();
-    defpol->d_kind = (DNSFilterEngine::PolicyKind)boost::get<uint32_t>(have["defpol"]);
-    defpol->setName(polName);
-    if (defpol->d_kind == DNSFilterEngine::PolicyKind::Custom) {
-      defpol->d_custom.push_back(DNSRecordContent::make(QType::CNAME, QClass::IN,
-                                                        boost::get<string>(have["defcontent"])));
+    params.defpol = DNSFilterEngine::Policy();
+    params.defpol->d_kind = (DNSFilterEngine::PolicyKind)boost::get<uint32_t>(have.at("defpol"));
+    params.defpol->setName(params.polName);
+    if (params.defpol->d_kind == DNSFilterEngine::PolicyKind::Custom) {
+      params.defcontent = boost::get<string>(have.at("defcontent"));
+      if (!params.defpol->d_custom) {
+        params.defpol->d_custom = make_unique<DNSFilterEngine::Policy::CustomData>();
+      }
+      params.defpol->d_custom->push_back(DNSRecordContent::make(QType::CNAME, QClass::IN,
+                                                                params.defcontent));
 
       if (have.count("defttl") != 0) {
-        defpol->d_ttl = static_cast<int32_t>(boost::get<uint32_t>(have["defttl"]));
+        params.defpol->d_ttl = static_cast<int32_t>(boost::get<uint32_t>(have.at("defttl")));
       }
       else {
-        defpol->d_ttl = -1; // get it from the zone
+        params.defpol->d_ttl = -1; // get it from the zone
       }
     }
 
     if (have.count("defpolOverrideLocalData") != 0) {
-      defpolOverrideLocal = boost::get<bool>(have["defpolOverrideLocalData"]);
+      params.defpolOverrideLocal = boost::get<bool>(have.at("defpolOverrideLocalData"));
     }
   }
   if (have.count("maxTTL") != 0) {
-    maxTTL = boost::get<uint32_t>(have["maxTTL"]);
+    params.maxTTL = boost::get<uint32_t>(have.at("maxTTL"));
   }
   if (have.count("zoneSizeHint") != 0) {
-    auto zoneSizeHint = static_cast<size_t>(boost::get<uint32_t>(have["zoneSizeHint"]));
-    if (zoneSizeHint > 0) {
-      zone->reserve(zoneSizeHint);
-    }
+    params.zoneSizeHint = static_cast<size_t>(boost::get<uint32_t>(have.at("zoneSizeHint")));
   }
   if (have.count("tags") != 0) {
-    const auto tagsTable = boost::get<std::vector<std::pair<int, std::string>>>(have["tags"]);
+    const auto& tagsTable = boost::get<std::vector<std::pair<int, std::string>>>(have.at("tags"));
     std::unordered_set<std::string> tags;
     for (const auto& tag : tagsTable) {
       tags.insert(tag.second);
+      params.tags.insert(tag.second);
     }
-    zone->setTags(std::move(tags));
   }
   if (have.count("overridesGettag") != 0) {
-    zone->setPolicyOverridesGettag(boost::get<bool>(have["overridesGettag"]));
+    params.defpolOverrideLocal = boost::get<bool>(have.at("overridesGettag"));
   }
   if (have.count("extendedErrorCode") != 0) {
-    auto code = boost::get<uint32_t>(have["extendedErrorCode"]);
+    auto code = boost::get<uint32_t>(have.at("extendedErrorCode"));
     if (code > std::numeric_limits<uint16_t>::max()) {
       throw std::runtime_error("Invalid extendedErrorCode value " + std::to_string(code) + " in RPZ configuration");
     }
-
-    zone->setExtendedErrorCode(static_cast<uint16_t>(code));
+    params.extendedErrorCode = code;
     if (have.count("extendedErrorExtra") != 0) {
-      zone->setExtendedErrorExtra(boost::get<std::string>(have["extendedErrorExtra"]));
+      params.extendedErrorExtra = boost::get<std::string>(have.at("extendedErrorExtra"));
     }
   }
   if (have.count("includeSOA") != 0) {
-    zone->setIncludeSOA(boost::get<bool>(have["includeSOA"]));
+    params.includeSOA = boost::get<bool>(have.at("includeSOA"));
   }
   if (have.count("ignoreDuplicates") != 0) {
-    zone->setIgnoreDuplicates(boost::get<bool>(have["ignoreDuplicates"]));
+    params.ignoreDuplicates = boost::get<bool>(have.at("ignoreDuplicates"));
   }
 }
 
 typedef std::unordered_map<std::string, boost::variant<bool, uint64_t, std::string, std::vector<std::pair<int, std::string>>>> protobufOptions_t;
 
-static void parseProtobufOptions(boost::optional<protobufOptions_t> vars, ProtobufExportConfig& config)
+static void parseProtobufOptions(const boost::optional<protobufOptions_t>& vars, ProtobufExportConfig& config)
 {
   if (!vars) {
     return;
   }
+  const auto& have = *vars;
 
-  if (vars->count("timeout")) {
-    config.timeout = boost::get<uint64_t>((*vars)["timeout"]);
+  if (have.count("timeout") != 0) {
+    config.timeout = boost::get<uint64_t>(have.at("timeout"));
   }
 
-  if (vars->count("maxQueuedEntries")) {
-    config.maxQueuedEntries = boost::get<uint64_t>((*vars)["maxQueuedEntries"]);
+  if (have.count("maxQueuedEntries") != 0) {
+    config.maxQueuedEntries = boost::get<uint64_t>(have.at("maxQueuedEntries"));
   }
 
-  if (vars->count("reconnectWaitTime")) {
-    config.reconnectWaitTime = boost::get<uint64_t>((*vars)["reconnectWaitTime"]);
+  if (have.count("reconnectWaitTime") != 0) {
+    config.reconnectWaitTime = boost::get<uint64_t>(have.at("reconnectWaitTime"));
   }
 
-  if (vars->count("asyncConnect")) {
-    config.asyncConnect = boost::get<bool>((*vars)["asyncConnect"]);
+  if (have.count("asyncConnect") != 0) {
+    config.asyncConnect = boost::get<bool>(have.at("asyncConnect"));
   }
 
-  if (vars->count("taggedOnly")) {
-    config.taggedOnly = boost::get<bool>((*vars)["taggedOnly"]);
+  if (have.count("taggedOnly") != 0) {
+    config.taggedOnly = boost::get<bool>(have.at("taggedOnly"));
   }
 
-  if (vars->count("logQueries")) {
-    config.logQueries = boost::get<bool>((*vars)["logQueries"]);
+  if (have.count("logQueries") != 0) {
+    config.logQueries = boost::get<bool>(have.at("logQueries"));
   }
 
-  if (vars->count("logResponses")) {
-    config.logResponses = boost::get<bool>((*vars)["logResponses"]);
+  if (have.count("logResponses") != 0) {
+    config.logResponses = boost::get<bool>(have.at("logResponses"));
   }
 
-  if (vars->count("logMappedFrom")) {
-    config.logMappedFrom = boost::get<bool>((*vars)["logMappedFrom"]);
+  if (have.count("logMappedFrom") != 0) {
+    config.logMappedFrom = boost::get<bool>(have.at("logMappedFrom"));
   }
 
-  if (vars->count("exportTypes")) {
+  if (have.count("exportTypes") != 0) {
     config.exportTypes.clear();
 
-    auto types = boost::get<std::vector<std::pair<int, std::string>>>((*vars)["exportTypes"]);
+    auto types = boost::get<std::vector<std::pair<int, std::string>>>(have.at("exportTypes"));
     for (const auto& pair : types) {
       const auto& type = pair.second;
 
@@ -228,155 +229,113 @@ static void parseProtobufOptions(boost::optional<protobufOptions_t> vars, Protob
 #ifdef HAVE_FSTRM
 typedef std::unordered_map<std::string, boost::variant<bool, uint64_t, std::string, std::vector<std::pair<int, std::string>>>> frameStreamOptions_t;
 
-static void parseFrameStreamOptions(boost::optional<frameStreamOptions_t> vars, FrameStreamExportConfig& config)
+static void parseFrameStreamOptions(const boost::optional<frameStreamOptions_t>& vars, FrameStreamExportConfig& config)
 {
   if (!vars) {
     return;
   }
+  const auto& have = *vars;
 
-  if (vars->count("logQueries")) {
-    config.logQueries = boost::get<bool>((*vars)["logQueries"]);
+  if (have.count("logQueries") != 0) {
+    config.logQueries = boost::get<bool>(have.at("logQueries"));
   }
-  if (vars->count("logResponses")) {
-    config.logResponses = boost::get<bool>((*vars)["logResponses"]);
+  if (have.count("logResponses") != 0) {
+    config.logResponses = boost::get<bool>(have.at("logResponses"));
   }
-  if (vars->count("logNODs")) {
-    config.logNODs = boost::get<bool>((*vars)["logNODs"]);
+  if (have.count("logNODs") != 0) {
+    config.logNODs = boost::get<bool>(have.at("logNODs"));
   }
-  if (vars->count("logUDRs")) {
-    config.logUDRs = boost::get<bool>((*vars)["logUDRs"]);
+  if (have.count("logUDRs") != 0) {
+    config.logUDRs = boost::get<bool>(have.at("logUDRs"));
   }
 
-  if (vars->count("bufferHint")) {
-    config.bufferHint = boost::get<uint64_t>((*vars)["bufferHint"]);
+  if (have.count("bufferHint") != 0) {
+    config.bufferHint = boost::get<uint64_t>(have.at("bufferHint"));
   }
-  if (vars->count("flushTimeout")) {
-    config.flushTimeout = boost::get<uint64_t>((*vars)["flushTimeout"]);
+  if (have.count("flushTimeout") != 0) {
+    config.flushTimeout = boost::get<uint64_t>(have.at("flushTimeout"));
   }
-  if (vars->count("inputQueueSize")) {
-    config.inputQueueSize = boost::get<uint64_t>((*vars)["inputQueueSize"]);
+  if (have.count("inputQueueSize") != 0) {
+    config.inputQueueSize = boost::get<uint64_t>(have.at("inputQueueSize"));
   }
-  if (vars->count("outputQueueSize")) {
-    config.outputQueueSize = boost::get<uint64_t>((*vars)["outputQueueSize"]);
+  if (have.count("outputQueueSize") != 0) {
+    config.outputQueueSize = boost::get<uint64_t>(have.at("outputQueueSize"));
   }
-  if (vars->count("queueNotifyThreshold")) {
-    config.queueNotifyThreshold = boost::get<uint64_t>((*vars)["queueNotifyThreshold"]);
+  if (have.count("queueNotifyThreshold") != 0) {
+    config.queueNotifyThreshold = boost::get<uint64_t>(have.at("queueNotifyThreshold"));
   }
-  if (vars->count("reopenInterval")) {
-    config.reopenInterval = boost::get<uint64_t>((*vars)["reopenInterval"]);
+  if (have.count("reopenInterval") != 0) {
+    config.reopenInterval = boost::get<uint64_t>(have.at("reopenInterval"));
   }
 }
 #endif /* HAVE_FSTRM */
 
-static void rpzPrimary(LuaConfigItems& lci, luaConfigDelayedThreads& delayedThreads, const boost::variant<string, std::vector<std::pair<int, string>>>& primaries_, const string& zoneName, boost::optional<rpzOptions_t> options)
+static void rpzPrimary(LuaConfigItems& lci, const boost::variant<string, std::vector<std::pair<int, string>>>& primaries_, const string& zoneName, const boost::optional<rpzOptions_t>& options)
 {
-  boost::optional<DNSFilterEngine::Policy> defpol;
-  bool defpolOverrideLocal = true;
+  RPZTrackerParams params;
+  params.name = zoneName;
+  params.polName = zoneName;
+
   std::shared_ptr<DNSFilterEngine::Zone> zone = std::make_shared<DNSFilterEngine::Zone>();
-  TSIGTriplet tt;
-  uint32_t refresh = 0;
-  size_t maxReceivedXFRMBytes = 0;
-  uint16_t axfrTimeout = 20;
-  uint32_t maxTTL = std::numeric_limits<uint32_t>::max();
-  ComboAddress localAddress;
-  std::vector<ComboAddress> primaries;
   if (primaries_.type() == typeid(string)) {
-    primaries.push_back(ComboAddress(boost::get<std::string>(primaries_), 53));
+    params.primaries.emplace_back(boost::get<std::string>(primaries_), 53);
   }
   else {
     for (const auto& primary : boost::get<std::vector<std::pair<int, std::string>>>(primaries_)) {
-      primaries.push_back(ComboAddress(primary.second, 53));
+      params.primaries.emplace_back(primary.second, 53);
     }
   }
 
-  size_t zoneIdx;
-  std::string dumpFile;
-  std::shared_ptr<const SOARecordContent> sr = nullptr;
-
   try {
-    std::string seedFile;
-    std::string polName(zoneName);
-
     if (options) {
       auto& have = *options;
-      parseRPZParameters(have, zone, polName, defpol, defpolOverrideLocal, maxTTL);
+      parseRPZParameters(have, params);
 
-      if (have.count("tsigname")) {
-        tt.name = DNSName(toLower(boost::get<string>(have["tsigname"])));
-        tt.algo = DNSName(toLower(boost::get<string>(have["tsigalgo"])));
-        if (B64Decode(boost::get<string>(have["tsigsecret"]), tt.secret))
+      if (have.count("tsigname") != 0) {
+        params.tsigtriplet.name = DNSName(toLower(boost::get<string>(have.at("tsigname"))));
+        params.tsigtriplet.algo = DNSName(toLower(boost::get<string>(have.at("tsigalgo"))));
+        if (B64Decode(boost::get<string>(have.at("tsigsecret")), params.tsigtriplet.secret) != 0) {
           throw std::runtime_error("TSIG secret is not valid Base-64 encoded");
+        }
       }
-
-      if (have.count("refresh")) {
-        refresh = boost::get<uint32_t>(have["refresh"]);
-        if (refresh == 0) {
+      if (have.count("refresh") != 0) {
+        params.refreshFromConf = boost::get<uint32_t>(have.at("refresh"));
+        if (params.refreshFromConf == 0) {
           SLOG(g_log << Logger::Warning << "rpzPrimary refresh value of 0 ignored" << endl,
                lci.d_slog->info(Logr::Warning, "rpzPrimary refresh value of 0 ignored"));
         }
       }
 
-      if (have.count("maxReceivedMBytes")) {
-        maxReceivedXFRMBytes = static_cast<size_t>(boost::get<uint32_t>(have["maxReceivedMBytes"]));
+      if (have.count("maxReceivedMBytes") != 0) {
+        params.maxReceivedMBytes = static_cast<size_t>(boost::get<uint32_t>(have.at("maxReceivedMBytes")));
       }
 
-      if (have.count("localAddress")) {
-        localAddress = ComboAddress(boost::get<string>(have["localAddress"]));
+      if (have.count("localAddress") != 0) {
+        params.localAddress = ComboAddress(boost::get<string>(have.at("localAddress")));
       }
 
-      if (have.count("axfrTimeout")) {
-        axfrTimeout = static_cast<uint16_t>(boost::get<uint32_t>(have["axfrTimeout"]));
+      if (have.count("axfrTimeout") != 0) {
+        params.xfrTimeout = static_cast<uint16_t>(boost::get<uint32_t>(have.at("axfrTimeout")));
       }
 
-      if (have.count("seedFile")) {
-        seedFile = boost::get<std::string>(have["seedFile"]);
+      if (have.count("seedFile") != 0) {
+        params.seedFileName = boost::get<std::string>(have.at("seedFile"));
       }
 
-      if (have.count("dumpFile")) {
-        dumpFile = boost::get<std::string>(have["dumpFile"]);
+      if (have.count("dumpFile") != 0) {
+        params.dumpZoneFileName = boost::get<std::string>(have.at("dumpFile"));
       }
     }
 
-    if (localAddress != ComboAddress()) {
+    if (params.localAddress != ComboAddress()) {
       // We were passed a localAddress, check if its AF matches the primaries'
-      for (const auto& primary : primaries) {
-        if (localAddress.sin4.sin_family != primary.sin4.sin_family) {
-          throw PDNSException("Primary address(" + primary.toString() + ") is not of the same Address Family as the local address (" + localAddress.toString() + ").");
+      for (const auto& primary : params.primaries) {
+        if (params.localAddress.sin4.sin_family != primary.sin4.sin_family) {
+          throw PDNSException("Primary address(" + primary.toString() + ") is not of the same Address Family as the local address (" + params.localAddress.toString() + ").");
         }
       }
     }
-
-    DNSName domain(zoneName);
-    zone->setDomain(domain);
-    zone->setName(polName);
-    zoneIdx = lci.dfe.addZone(zone);
-
-    auto log = lci.d_slog->withValues("seedfile", Logging::Loggable(seedFile), "zone", Logging::Loggable(zoneName));
-    if (!seedFile.empty()) {
-      SLOG(g_log << Logger::Info << "Pre-loading RPZ zone " << zoneName << " from seed file '" << seedFile << "'" << endl,
-           log->info(Logr::Info, "Pre-loading RPZ zone from seed file"));
-      try {
-        sr = loadRPZFromFile(seedFile, zone, defpol, defpolOverrideLocal, maxTTL);
-
-        if (zone->getDomain() != domain) {
-          throw PDNSException("The RPZ zone " + zoneName + " loaded from the seed file (" + zone->getDomain().toString() + ") does not match the one passed in parameter (" + domain.toString() + ")");
-        }
-
-        if (sr == nullptr) {
-          throw PDNSException("The RPZ zone " + zoneName + " loaded from the seed file (" + zone->getDomain().toString() + ") has no SOA record");
-        }
-      }
-      catch (const PDNSException& e) {
-        SLOG(g_log << Logger::Warning << "Unable to pre-load RPZ zone " << zoneName << " from seed file '" << seedFile << "': " << e.reason << endl,
-             log->error(Logr::Warning, e.reason, "Exception while pre-loading RPZ zone", "exception", Logging::Loggable("PDNSException")));
-        zone->clear();
-      }
-      catch (const std::exception& e) {
-        SLOG(g_log << Logger::Warning << "Unable to pre-load RPZ zone " << zoneName << " from seed file '" << seedFile << "': " << e.what() << endl,
-             log->error(Logr::Warning, e.what(), "Exception while pre-loading RPZ zone", "exception", Logging::Loggable("std::exception")));
-        zone->clear();
-      }
-    }
+    lci.rpzs.emplace_back(params);
   }
   catch (const std::exception& e) {
     SLOG(g_log << Logger::Error << "Problem configuring 'rpzPrimary': " << e.what() << endl,
@@ -386,8 +345,6 @@ static void rpzPrimary(LuaConfigItems& lci, luaConfigDelayedThreads& delayedThre
     SLOG(g_log << Logger::Error << "Problem configuring 'rpzPrimary': " << e.reason << endl,
          lci.d_slog->error(Logr::Error, e.reason, "Exception configuring 'rpzPrimary'", Logging::Loggable("PDNSException")));
   }
-
-  delayedThreads.rpzPrimaryThreads.emplace_back(primaries, defpol, defpolOverrideLocal, maxTTL, zoneIdx, tt, maxReceivedXFRMBytes, localAddress, axfrTimeout, refresh, sr, dumpFile);
 }
 
 // A wrapper class that loads the standard Lua defintions into the context, so that we can use things like pdns.A
@@ -418,10 +375,12 @@ public:
   }
 };
 
-void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& delayedThreads, ProxyMapping& proxyMapping)
+void loadRecursorLuaConfig(const std::string& fname, ProxyMapping& proxyMapping, LuaConfigItems& newLuaConfig) // NOLINT(readability-function-cognitive-complexity)
 {
   LuaConfigItems lci;
-  lci.d_slog = g_slog->withName("luaconfig");
+  if (g_slog) {
+    lci.d_slog = g_slog->withName("luaconfig");
+  }
 
   RecLuaConfigContext Lua;
 
@@ -451,38 +410,22 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
   Lua->writeVariable("Policy", pmap);
 
   Lua->writeFunction("rpzFile", [&lci](const string& filename, boost::optional<rpzOptions_t> options) {
-    auto log = lci.d_slog->withValues("file", Logging::Loggable(filename));
-    try {
-      boost::optional<DNSFilterEngine::Policy> defpol;
-      bool defpolOverrideLocal = true;
-      std::string polName("rpzFile");
-      std::shared_ptr<DNSFilterEngine::Zone> zone = std::make_shared<DNSFilterEngine::Zone>();
-      uint32_t maxTTL = std::numeric_limits<uint32_t>::max();
-      if (options) {
-        auto& have = *options;
-        parseRPZParameters(have, zone, polName, defpol, defpolOverrideLocal, maxTTL);
-      }
-      SLOG(g_log << Logger::Warning << "Loading RPZ from file '" << filename << "'" << endl,
-           log->info(Logr::Info, "Loading RPZ from file"));
-      zone->setName(polName);
-      loadRPZFromFile(filename, zone, defpol, defpolOverrideLocal, maxTTL);
-      lci.dfe.addZone(zone);
-      SLOG(g_log << Logger::Warning << "Done loading RPZ from file '" << filename << "'" << endl,
-           log->info(Logr::Info,  "Done loading RPZ from file"));
-    }
-    catch (const std::exception& e) {
-      SLOG(g_log << Logger::Error << "Unable to load RPZ zone from '" << filename << "': " << e.what() << endl,
-           log->error(Logr::Error, e.what(), "Exception while loading RPZ zone from file"));
+    RPZTrackerParams params;
+    params.name = filename;
+    params.polName = "rpzFile";
+    if (options) {
+      parseRPZParameters(*options, params);
     }
+    lci.rpzs.emplace_back(params);
   });
 
-  Lua->writeFunction("rpzMaster", [&lci, &delayedThreads](const boost::variant<string, std::vector<std::pair<int, string>>>& primaries_, const string& zoneName, boost::optional<rpzOptions_t> options) {
+  Lua->writeFunction("rpzMaster", [&lci](const boost::variant<string, std::vector<std::pair<int, string>>>& primaries_, const string& zoneName, const boost::optional<rpzOptions_t>& options) {
     SLOG(g_log << Logger::Warning << "'rpzMaster' is deprecated and will be removed in a future release, use 'rpzPrimary' instead" << endl,
          lci.d_slog->info(Logr::Warning, "'rpzMaster' is deprecated and will be removed in a future release, use 'rpzPrimary' instead"));
-    rpzPrimary(lci, delayedThreads, primaries_, zoneName, options);
+    rpzPrimary(lci, primaries_, zoneName, options);
   });
-  Lua->writeFunction("rpzPrimary", [&lci, &delayedThreads](const boost::variant<string, std::vector<std::pair<int, string>>>& primaries_, const string& zoneName, boost::optional<rpzOptions_t> options) {
-    rpzPrimary(lci, delayedThreads, primaries_, zoneName, options);
+  Lua->writeFunction("rpzPrimary", [&lci](const boost::variant<string, std::vector<std::pair<int, string>>>& primaries_, const string& zoneName, const boost::optional<rpzOptions_t>& options) {
+    rpzPrimary(lci, primaries_, zoneName, options);
   });
 
   typedef std::unordered_map<std::string, boost::variant<uint32_t, std::string>> zoneToCacheOptions_t;
@@ -560,7 +503,7 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
         }
       }
 
-      lci.ztcConfigs[validZoneName] = conf;
+      lci.ztcConfigs[validZoneName] = std::move(conf);
     }
     catch (const std::exception& e) {
       SLOG(g_log << Logger::Error << "Problem configuring zoneToCache for zone '" << zoneName << "': " << e.what() << endl,
@@ -602,14 +545,12 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
                      });
 
   Lua->writeFunction("addTA", [&lci](const std::string& who, const std::string& what) {
-    warnIfDNSSECDisabled("Warning: adding Trust Anchor for DNSSEC (addTA), but dnssec is set to 'off'!");
     DNSName zone(who);
     auto ds = std::dynamic_pointer_cast<DSRecordContent>(DSRecordContent::make(what));
     lci.dsAnchors[zone].insert(*ds);
   });
 
   Lua->writeFunction("clearTA", [&lci](boost::optional<string> who) {
-    warnIfDNSSECDisabled("Warning: removing Trust Anchor for DNSSEC (clearTA), but dnssec is set to 'off'!");
     if (who)
       lci.dsAnchors.erase(DNSName(*who));
     else
@@ -618,7 +559,6 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
 
   /* Remove in 4.3 */
   Lua->writeFunction("addDS", [&lci](const std::string& who, const std::string& what) {
-    warnIfDNSSECDisabled("Warning: adding Trust Anchor for DNSSEC (addDS), but dnssec is set to 'off'!");
     SLOG(g_log << Logger::Warning << "addDS is deprecated and will be removed in the future, switch to addTA" << endl,
          lci.d_slog->info(Logr::Warning, "addDS is deprecated and will be removed in the future, switch to addTA"));
     DNSName zone(who);
@@ -630,7 +570,6 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
   Lua->writeFunction("clearDS", [&lci](boost::optional<string> who) {
     SLOG(g_log << Logger::Warning << "clearDS is deprecated and will be removed in the future, switch to clearTA" << endl,
          lci.d_slog->info(Logr::Warning, "clearDS is deprecated and will be removed in the future, switch to clearTA"));
-    warnIfDNSSECDisabled("Warning: removing Trust Anchor for DNSSEC (clearDS), but dnssec is set to 'off'!");
     if (who)
       lci.dsAnchors.erase(DNSName(*who));
     else
@@ -638,7 +577,6 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
   });
 
   Lua->writeFunction("addNTA", [&lci](const std::string& who, const boost::optional<std::string> why) {
-    warnIfDNSSECDisabled("Warning: adding Negative Trust Anchor for DNSSEC (addNTA), but dnssec is set to 'off'!");
     if (why)
       lci.negAnchors[DNSName(who)] = static_cast<string>(*why);
     else
@@ -646,7 +584,6 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
   });
 
   Lua->writeFunction("clearNTA", [&lci](boost::optional<string> who) {
-    warnIfDNSSECDisabled("Warning: removing Negative Trust Anchor for DNSSEC (clearNTA), but dnssec is set to 'off'!");
     if (who)
       lci.negAnchors.erase(DNSName(*who));
     else
@@ -658,10 +595,8 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
     if (interval) {
       realInterval = static_cast<uint32_t>(*interval);
     }
-    warnIfDNSSECDisabled("Warning: reading Trust Anchors from file (readTrustAnchorsFromFile), but dnssec is set to 'off'!");
     lci.trustAnchorFileInfo.fname = fnamearg;
     lci.trustAnchorFileInfo.interval = realInterval;
-    updateTrustAnchorsFromFile(fnamearg, lci.dsAnchors, lci.d_slog);
   });
 
   Lua->writeFunction("setProtobufMasks", [&lci](const uint8_t maskV4, uint8_t maskV6) {
@@ -871,7 +806,7 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
 
   try {
     Lua->executeCode(ifs);
-    g_luaconfs.setState(std::move(lci));
+    newLuaConfig = lci;
   }
   catch (const LuaContext::ExecutionErrorException& e) {
     SLOG(g_log << Logger::Error << "Unable to load Lua script from '" + fname + "': ",
@@ -897,25 +832,3 @@ void loadRecursorLuaConfig(const std::string& fname, luaConfigDelayedThreads& de
     throw;
   }
 }
-
-void startLuaConfigDelayedThreads(const luaConfigDelayedThreads& delayedThreads, uint64_t generation)
-{
-  for (const auto& rpzPrimary : delayedThreads.rpzPrimaryThreads) {
-    try {
-      // The get calls all return a value object here. That is essential, since we want copies so that RPZIXFRTracker gets values
-      // with the proper lifetime.
-      std::thread t(RPZIXFRTracker, std::get<0>(rpzPrimary), std::get<1>(rpzPrimary), std::get<2>(rpzPrimary), std::get<3>(rpzPrimary), std::get<4>(rpzPrimary), std::get<5>(rpzPrimary), std::get<6>(rpzPrimary) * 1024 * 1024, std::get<7>(rpzPrimary), std::get<8>(rpzPrimary), std::get<9>(rpzPrimary), std::get<10>(rpzPrimary), std::get<11>(rpzPrimary), generation);
-      t.detach();
-    }
-    catch (const std::exception& e) {
-      SLOG(g_log << Logger::Error << "Problem starting RPZIXFRTracker thread: " << e.what() << endl,
-           g_slog->withName("rpz")->error(Logr::Error, e.what(), "Exception starting RPZIXFRTracker thread", "exception", Logging::Loggable("std::exception")));
-      exit(1);
-    }
-    catch (const PDNSException& e) {
-      SLOG(g_log << Logger::Error << "Problem starting RPZIXFRTracker thread: " << e.reason << endl,
-           g_slog->withName("rpz")->error(Logr::Error, e.reason, "Exception starting RPZIXFRTracker thread", "exception", Logging::Loggable("PDNSException")));
-      exit(1);
-    }
-  }
-}
